@@ -13,7 +13,7 @@ double cpp_prior_b(Rcpp::List param, Rcpp::List config,
                    Rcpp::RObject custom_function);
 
 double cpp_prior_all(Rcpp::List param, Rcpp::List config,
-                     Rcpp::RObject custom_functions = R_NilValue);
+                     Rcpp::RObject custom_functions);
 
 
 #endif
